# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from typing_extensions import Literal

import httpx

from ..types import webhook_test_params, webhook_create_params, webhook_update_params, webhook_list_deliveries_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.webhook_list_response import WebhookListResponse
from ..types.webhook_test_response import WebhookTestResponse
from ..types.webhook_create_response import WebhookCreateResponse
from ..types.webhook_delete_response import WebhookDeleteResponse
from ..types.webhook_update_response import WebhookUpdateResponse
from ..types.webhook_retrieve_response import WebhookRetrieveResponse
from ..types.webhook_list_deliveries_response import WebhookListDeliveriesResponse
from ..types.webhook_replay_delivery_response import WebhookReplayDeliveryResponse
from ..types.webhook_retrieve_delivery_response import WebhookRetrieveDeliveryResponse

__all__ = ["WebhooksResource", "AsyncWebhooksResource"]


class WebhooksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return WebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return WebhooksResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        url: str,
        all_events: Optional[bool] | Omit = omit,
        enabled: Optional[bool] | Omit = omit,
        events: Optional[
            List[
                Literal[
                    "MessageSent",
                    "MessageDelayed",
                    "MessageDeliveryFailed",
                    "MessageHeld",
                    "MessageBounced",
                    "MessageLinkClicked",
                    "MessageLoaded",
                    "DomainDNSError",
                ]
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookCreateResponse:
        """
        Create a webhook endpoint to receive email event notifications.

        **Available events:**

        - `MessageSent` - Email accepted by recipient server
        - `MessageDeliveryFailed` - Delivery permanently failed
        - `MessageDelayed` - Delivery temporarily failed, will retry
        - `MessageBounced` - Email bounced
        - `MessageHeld` - Email held for review
        - `MessageLinkClicked` - Recipient clicked a link
        - `MessageLoaded` - Recipient opened the email
        - `DomainDNSError` - Domain DNS issue detected

        Args:
          name: Webhook name for identification

          url: HTTPS endpoint URL

          all_events: Subscribe to all events (ignores events array, accepts null)

          enabled: Whether the webhook is enabled (accepts null)

          events:
              Events to subscribe to (accepts null):

              - `MessageSent` - Email successfully delivered to recipient's server
              - `MessageDelayed` - Temporary delivery failure, will retry
              - `MessageDeliveryFailed` - Permanent delivery failure
              - `MessageHeld` - Email held for manual review
              - `MessageBounced` - Email bounced back
              - `MessageLinkClicked` - Recipient clicked a tracked link
              - `MessageLoaded` - Recipient opened the email (tracking pixel loaded)
              - `DomainDNSError` - DNS configuration issue detected

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/webhooks",
            body=maybe_transform(
                {
                    "name": name,
                    "url": url,
                    "all_events": all_events,
                    "enabled": enabled,
                    "events": events,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateResponse,
        )

    def retrieve(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookRetrieveResponse:
        """
        Get webhook details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._get(
            f"/webhooks/{webhook_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookRetrieveResponse,
        )

    def update(
        self,
        webhook_id: str,
        *,
        all_events: Optional[bool] | Omit = omit,
        enabled: Optional[bool] | Omit = omit,
        events: Optional[SequenceNotStr[str]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookUpdateResponse:
        """
        Update a webhook

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._patch(
            f"/webhooks/{webhook_id}",
            body=maybe_transform(
                {
                    "all_events": all_events,
                    "enabled": enabled,
                    "events": events,
                    "name": name,
                    "url": url,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListResponse:
        """Get all configured webhook endpoints"""
        return self._get(
            "/webhooks",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookListResponse,
        )

    def delete(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeleteResponse:
        """
        Delete a webhook

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._delete(
            f"/webhooks/{webhook_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeleteResponse,
        )

    def list_deliveries(
        self,
        webhook_id: str,
        *,
        after: int | Omit = omit,
        before: int | Omit = omit,
        event: Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ]
        | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        success: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListDeliveriesResponse:
        """
        Get a paginated list of delivery attempts for a specific webhook.

        Use this to:

        - Monitor webhook health and delivery success rate
        - Debug failed deliveries
        - Find specific events to replay

        **Filtering:**

        - Filter by success/failure to find problematic deliveries
        - Filter by event type to find specific events
        - Filter by time range for debugging recent issues

        **Retry behavior:** Failed deliveries are automatically retried with exponential
        backoff over ~3 days. Check `willRetry` to see if more attempts are scheduled.

        Args:
          after: Only deliveries after this Unix timestamp

          before: Only deliveries before this Unix timestamp

          event: Filter by event type

          page: Page number (default 1)

          per_page: Items per page (default 30, max 100)

          success: Filter by delivery success (true = 2xx response, false = non-2xx or error)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._get(
            f"/webhooks/{webhook_id}/deliveries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "event": event,
                        "page": page,
                        "per_page": per_page,
                        "success": success,
                    },
                    webhook_list_deliveries_params.WebhookListDeliveriesParams,
                ),
            ),
            cast_to=WebhookListDeliveriesResponse,
        )

    def replay_delivery(
        self,
        delivery_id: str,
        *,
        webhook_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReplayDeliveryResponse:
        """
        Re-send a webhook delivery to your endpoint.

        **Use cases:**

        - Recover from transient failures after fixing your endpoint
        - Test endpoint changes with real historical data
        - Retry deliveries that failed due to downtime

        **How it works:**

        1. Fetches the original payload from the delivery
        2. Generates a new timestamp and signature
        3. Sends to your webhook URL immediately
        4. Returns the result (does not queue for retry if it fails)

        **Note:** The webhook must be enabled to replay deliveries.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        if not delivery_id:
            raise ValueError(f"Expected a non-empty value for `delivery_id` but received {delivery_id!r}")
        return self._post(
            f"/webhooks/{webhook_id}/deliveries/{delivery_id}/replay",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReplayDeliveryResponse,
        )

    def retrieve_delivery(
        self,
        delivery_id: str,
        *,
        webhook_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookRetrieveDeliveryResponse:
        """
        Get detailed information about a specific webhook delivery attempt.

        Returns:

        - The complete request payload that was sent
        - Request headers including the signature
        - Response status code and body from your endpoint
        - Timing information

        Use this to debug why a delivery failed or verify what data was sent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        if not delivery_id:
            raise ValueError(f"Expected a non-empty value for `delivery_id` but received {delivery_id!r}")
        return self._get(
            f"/webhooks/{webhook_id}/deliveries/{delivery_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookRetrieveDeliveryResponse,
        )

    def test(
        self,
        webhook_id: str,
        *,
        event: Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookTestResponse:
        """
        Send a test payload to your webhook endpoint and verify it receives the data
        correctly.

        Use this to:

        - Verify your webhook URL is accessible
        - Test your signature verification code
        - Ensure your server handles the payload format correctly

        **Test payload format:** The test payload is identical to real webhook payloads,
        containing sample data for the specified event type. Your webhook should respond
        with a 2xx status code.

        Args:
          event: Event type to simulate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return self._post(
            f"/webhooks/{webhook_id}/test",
            body=maybe_transform({"event": event}, webhook_test_params.WebhookTestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookTestResponse,
        )


class AsyncWebhooksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebhooksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#accessing-raw-response-data-eg-headers
        """
        return AsyncWebhooksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebhooksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/ArkHQ-io/ark-python#with_streaming_response
        """
        return AsyncWebhooksResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        url: str,
        all_events: Optional[bool] | Omit = omit,
        enabled: Optional[bool] | Omit = omit,
        events: Optional[
            List[
                Literal[
                    "MessageSent",
                    "MessageDelayed",
                    "MessageDeliveryFailed",
                    "MessageHeld",
                    "MessageBounced",
                    "MessageLinkClicked",
                    "MessageLoaded",
                    "DomainDNSError",
                ]
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookCreateResponse:
        """
        Create a webhook endpoint to receive email event notifications.

        **Available events:**

        - `MessageSent` - Email accepted by recipient server
        - `MessageDeliveryFailed` - Delivery permanently failed
        - `MessageDelayed` - Delivery temporarily failed, will retry
        - `MessageBounced` - Email bounced
        - `MessageHeld` - Email held for review
        - `MessageLinkClicked` - Recipient clicked a link
        - `MessageLoaded` - Recipient opened the email
        - `DomainDNSError` - Domain DNS issue detected

        Args:
          name: Webhook name for identification

          url: HTTPS endpoint URL

          all_events: Subscribe to all events (ignores events array, accepts null)

          enabled: Whether the webhook is enabled (accepts null)

          events:
              Events to subscribe to (accepts null):

              - `MessageSent` - Email successfully delivered to recipient's server
              - `MessageDelayed` - Temporary delivery failure, will retry
              - `MessageDeliveryFailed` - Permanent delivery failure
              - `MessageHeld` - Email held for manual review
              - `MessageBounced` - Email bounced back
              - `MessageLinkClicked` - Recipient clicked a tracked link
              - `MessageLoaded` - Recipient opened the email (tracking pixel loaded)
              - `DomainDNSError` - DNS configuration issue detected

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/webhooks",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "url": url,
                    "all_events": all_events,
                    "enabled": enabled,
                    "events": events,
                },
                webhook_create_params.WebhookCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookCreateResponse,
        )

    async def retrieve(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookRetrieveResponse:
        """
        Get webhook details

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._get(
            f"/webhooks/{webhook_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookRetrieveResponse,
        )

    async def update(
        self,
        webhook_id: str,
        *,
        all_events: Optional[bool] | Omit = omit,
        enabled: Optional[bool] | Omit = omit,
        events: Optional[SequenceNotStr[str]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookUpdateResponse:
        """
        Update a webhook

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._patch(
            f"/webhooks/{webhook_id}",
            body=await async_maybe_transform(
                {
                    "all_events": all_events,
                    "enabled": enabled,
                    "events": events,
                    "name": name,
                    "url": url,
                },
                webhook_update_params.WebhookUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListResponse:
        """Get all configured webhook endpoints"""
        return await self._get(
            "/webhooks",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookListResponse,
        )

    async def delete(
        self,
        webhook_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookDeleteResponse:
        """
        Delete a webhook

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._delete(
            f"/webhooks/{webhook_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookDeleteResponse,
        )

    async def list_deliveries(
        self,
        webhook_id: str,
        *,
        after: int | Omit = omit,
        before: int | Omit = omit,
        event: Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ]
        | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        success: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookListDeliveriesResponse:
        """
        Get a paginated list of delivery attempts for a specific webhook.

        Use this to:

        - Monitor webhook health and delivery success rate
        - Debug failed deliveries
        - Find specific events to replay

        **Filtering:**

        - Filter by success/failure to find problematic deliveries
        - Filter by event type to find specific events
        - Filter by time range for debugging recent issues

        **Retry behavior:** Failed deliveries are automatically retried with exponential
        backoff over ~3 days. Check `willRetry` to see if more attempts are scheduled.

        Args:
          after: Only deliveries after this Unix timestamp

          before: Only deliveries before this Unix timestamp

          event: Filter by event type

          page: Page number (default 1)

          per_page: Items per page (default 30, max 100)

          success: Filter by delivery success (true = 2xx response, false = non-2xx or error)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._get(
            f"/webhooks/{webhook_id}/deliveries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "event": event,
                        "page": page,
                        "per_page": per_page,
                        "success": success,
                    },
                    webhook_list_deliveries_params.WebhookListDeliveriesParams,
                ),
            ),
            cast_to=WebhookListDeliveriesResponse,
        )

    async def replay_delivery(
        self,
        delivery_id: str,
        *,
        webhook_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookReplayDeliveryResponse:
        """
        Re-send a webhook delivery to your endpoint.

        **Use cases:**

        - Recover from transient failures after fixing your endpoint
        - Test endpoint changes with real historical data
        - Retry deliveries that failed due to downtime

        **How it works:**

        1. Fetches the original payload from the delivery
        2. Generates a new timestamp and signature
        3. Sends to your webhook URL immediately
        4. Returns the result (does not queue for retry if it fails)

        **Note:** The webhook must be enabled to replay deliveries.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        if not delivery_id:
            raise ValueError(f"Expected a non-empty value for `delivery_id` but received {delivery_id!r}")
        return await self._post(
            f"/webhooks/{webhook_id}/deliveries/{delivery_id}/replay",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookReplayDeliveryResponse,
        )

    async def retrieve_delivery(
        self,
        delivery_id: str,
        *,
        webhook_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookRetrieveDeliveryResponse:
        """
        Get detailed information about a specific webhook delivery attempt.

        Returns:

        - The complete request payload that was sent
        - Request headers including the signature
        - Response status code and body from your endpoint
        - Timing information

        Use this to debug why a delivery failed or verify what data was sent.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        if not delivery_id:
            raise ValueError(f"Expected a non-empty value for `delivery_id` but received {delivery_id!r}")
        return await self._get(
            f"/webhooks/{webhook_id}/deliveries/{delivery_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookRetrieveDeliveryResponse,
        )

    async def test(
        self,
        webhook_id: str,
        *,
        event: Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebhookTestResponse:
        """
        Send a test payload to your webhook endpoint and verify it receives the data
        correctly.

        Use this to:

        - Verify your webhook URL is accessible
        - Test your signature verification code
        - Ensure your server handles the payload format correctly

        **Test payload format:** The test payload is identical to real webhook payloads,
        containing sample data for the specified event type. Your webhook should respond
        with a 2xx status code.

        Args:
          event: Event type to simulate

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not webhook_id:
            raise ValueError(f"Expected a non-empty value for `webhook_id` but received {webhook_id!r}")
        return await self._post(
            f"/webhooks/{webhook_id}/test",
            body=await async_maybe_transform({"event": event}, webhook_test_params.WebhookTestParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=WebhookTestResponse,
        )


class WebhooksResourceWithRawResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = to_raw_response_wrapper(
            webhooks.create,
        )
        self.retrieve = to_raw_response_wrapper(
            webhooks.retrieve,
        )
        self.update = to_raw_response_wrapper(
            webhooks.update,
        )
        self.list = to_raw_response_wrapper(
            webhooks.list,
        )
        self.delete = to_raw_response_wrapper(
            webhooks.delete,
        )
        self.list_deliveries = to_raw_response_wrapper(
            webhooks.list_deliveries,
        )
        self.replay_delivery = to_raw_response_wrapper(
            webhooks.replay_delivery,
        )
        self.retrieve_delivery = to_raw_response_wrapper(
            webhooks.retrieve_delivery,
        )
        self.test = to_raw_response_wrapper(
            webhooks.test,
        )


class AsyncWebhooksResourceWithRawResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = async_to_raw_response_wrapper(
            webhooks.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            webhooks.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            webhooks.update,
        )
        self.list = async_to_raw_response_wrapper(
            webhooks.list,
        )
        self.delete = async_to_raw_response_wrapper(
            webhooks.delete,
        )
        self.list_deliveries = async_to_raw_response_wrapper(
            webhooks.list_deliveries,
        )
        self.replay_delivery = async_to_raw_response_wrapper(
            webhooks.replay_delivery,
        )
        self.retrieve_delivery = async_to_raw_response_wrapper(
            webhooks.retrieve_delivery,
        )
        self.test = async_to_raw_response_wrapper(
            webhooks.test,
        )


class WebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: WebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = to_streamed_response_wrapper(
            webhooks.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            webhooks.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            webhooks.update,
        )
        self.list = to_streamed_response_wrapper(
            webhooks.list,
        )
        self.delete = to_streamed_response_wrapper(
            webhooks.delete,
        )
        self.list_deliveries = to_streamed_response_wrapper(
            webhooks.list_deliveries,
        )
        self.replay_delivery = to_streamed_response_wrapper(
            webhooks.replay_delivery,
        )
        self.retrieve_delivery = to_streamed_response_wrapper(
            webhooks.retrieve_delivery,
        )
        self.test = to_streamed_response_wrapper(
            webhooks.test,
        )


class AsyncWebhooksResourceWithStreamingResponse:
    def __init__(self, webhooks: AsyncWebhooksResource) -> None:
        self._webhooks = webhooks

        self.create = async_to_streamed_response_wrapper(
            webhooks.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            webhooks.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            webhooks.update,
        )
        self.list = async_to_streamed_response_wrapper(
            webhooks.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            webhooks.delete,
        )
        self.list_deliveries = async_to_streamed_response_wrapper(
            webhooks.list_deliveries,
        )
        self.replay_delivery = async_to_streamed_response_wrapper(
            webhooks.replay_delivery,
        )
        self.retrieve_delivery = async_to_streamed_response_wrapper(
            webhooks.retrieve_delivery,
        )
        self.test = async_to_streamed_response_wrapper(
            webhooks.test,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["WebhookListDeliveriesParams"]


class WebhookListDeliveriesParams(TypedDict, total=False):
    after: int
    """Only deliveries after this Unix timestamp"""

    before: int
    """Only deliveries before this Unix timestamp"""

    event: Literal[
        "MessageSent",
        "MessageDelayed",
        "MessageDeliveryFailed",
        "MessageHeld",
        "MessageBounced",
        "MessageLinkClicked",
        "MessageLoaded",
        "DomainDNSError",
    ]
    """Filter by event type"""

    page: int
    """Page number (default 1)"""

    per_page: Annotated[int, PropertyInfo(alias="perPage")]
    """Items per page (default 30, max 100)"""

    success: bool
    """Filter by delivery success (true = 2xx response, false = non-2xx or error)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["EmailSendParams", "Attachment"]


class EmailSendParams(TypedDict, total=False):
    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Sender email address. Must be from a verified domain OR use sandbox mode.

    **Supported formats:**

    - Email only: `hello@yourdomain.com`
    - With display name: `Acme <hello@yourdomain.com>`
    - With quoted name: `"Acme Support" <support@yourdomain.com>`

    The domain portion must match a verified sending domain in your account.

    **Sandbox mode:** Use `sandbox@arkhq.io` to send test emails without domain
    verification. Sandbox emails can only be sent to organization members and are
    limited to 10 per day.
    """

    subject: Required[str]
    """Email subject line"""

    to: Required[SequenceNotStr[str]]
    """Recipient email addresses (max 50)"""

    attachments: Optional[Iterable[Attachment]]
    """File attachments (accepts null)"""

    bcc: Optional[SequenceNotStr[str]]
    """BCC recipients (accepts null)"""

    cc: Optional[SequenceNotStr[str]]
    """CC recipients (accepts null)"""

    headers: Optional[Dict[str, str]]
    """Custom email headers (accepts null)"""

    html: Optional[str]
    """HTML body content (accepts null). Maximum 5MB (5,242,880 characters).

    Combined with attachments, the total message must not exceed 14MB.
    """

    metadata: Optional[Dict[str, str]]
    """Custom key-value pairs attached to an email for webhook correlation.

    When you send an email with metadata, these key-value pairs are:

    - **Stored** with the message
    - **Returned** in all webhook event payloads (MessageSent, MessageBounced, etc.)
    - **Never visible** to email recipients

    This is useful for correlating webhook events with your internal systems (e.g.,
    user IDs, order IDs, campaign identifiers).

    **Validation Rules:**

    - Maximum 10 keys per email
    - Keys: 1-40 characters, must start with a letter, only alphanumeric and
      underscores (`^[a-zA-Z][a-zA-Z0-9_]*$`)
    - Values: 1-500 characters, no control characters (newlines, tabs, etc.)
    - Total size: 4KB maximum (JSON-encoded)
    """

    reply_to: Annotated[Optional[str], PropertyInfo(alias="replyTo")]
    """Reply-to address (accepts null)"""

    tag: Optional[str]
    """Tag for categorization and filtering (accepts null)"""

    text: Optional[str]
    """
    Plain text body (accepts null, auto-generated from HTML if not provided).
    Maximum 5MB (5,242,880 characters).
    """

    idempotency_key: Annotated[str, PropertyInfo(alias="Idempotency-Key")]


class Attachment(TypedDict, total=False):
    content: Required[str]
    """Base64-encoded file content"""

    content_type: Required[Annotated[str, PropertyInfo(alias="contentType")]]
    """MIME type"""

    filename: Required[str]
    """Attachment filename"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["WebhookUpdateParams"]


class WebhookUpdateParams(TypedDict, total=False):
    all_events: Annotated[Optional[bool], PropertyInfo(alias="allEvents")]

    enabled: Optional[bool]

    events: Optional[SequenceNotStr[str]]

    name: Optional[str]

    url: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from ark import Ark, AsyncArk
from ark.types import (
    WebhookListResponse,
    WebhookTestResponse,
    WebhookCreateResponse,
    WebhookDeleteResponse,
    WebhookUpdateResponse,
    WebhookRetrieveResponse,
    WebhookListDeliveriesResponse,
    WebhookReplayDeliveryResponse,
    WebhookRetrieveDeliveryResponse,
)
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWebhooks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Ark) -> None:
        webhook = client.webhooks.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        )
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Ark) -> None:
        webhook = client.webhooks.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
            all_events=True,
            enabled=True,
            events=["MessageSent", "MessageDeliveryFailed", "MessageBounced"],
        )
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Ark) -> None:
        webhook = client.webhooks.retrieve(
            "webhookId",
        )
        assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.retrieve(
            "webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.retrieve(
            "webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Ark) -> None:
        webhook = client.webhooks.update(
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Ark) -> None:
        webhook = client.webhooks.update(
            webhook_id="webhookId",
            all_events=True,
            enabled=True,
            events=["string"],
            name="name",
            url="https://example.com",
        )
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.update(
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.update(
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.update(
                webhook_id="",
            )

    @parametrize
    def test_method_list(self, client: Ark) -> None:
        webhook = client.webhooks.list()
        assert_matches_type(WebhookListResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookListResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookListResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Ark) -> None:
        webhook = client.webhooks.delete(
            "webhookId",
        )
        assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.delete(
            "webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.delete(
            "webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_list_deliveries(self, client: Ark) -> None:
        webhook = client.webhooks.list_deliveries(
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    def test_method_list_deliveries_with_all_params(self, client: Ark) -> None:
        webhook = client.webhooks.list_deliveries(
            webhook_id="webhookId",
            after=0,
            before=0,
            event="MessageSent",
            page=1,
            per_page=1,
            success=True,
        )
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_list_deliveries(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.list_deliveries(
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_list_deliveries(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.list_deliveries(
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_deliveries(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.list_deliveries(
                webhook_id="",
            )

    @parametrize
    def test_method_replay_delivery(self, client: Ark) -> None:
        webhook = client.webhooks.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_replay_delivery(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_replay_delivery(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_replay_delivery(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.replay_delivery(
                delivery_id="deliveryId",
                webhook_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `delivery_id` but received ''"):
            client.webhooks.with_raw_response.replay_delivery(
                delivery_id="",
                webhook_id="webhookId",
            )

    @parametrize
    def test_method_retrieve_delivery(self, client: Ark) -> None:
        webhook = client.webhooks.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_retrieve_delivery(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_delivery(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_delivery(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.retrieve_delivery(
                delivery_id="deliveryId",
                webhook_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `delivery_id` but received ''"):
            client.webhooks.with_raw_response.retrieve_delivery(
                delivery_id="",
                webhook_id="webhookId",
            )

    @parametrize
    def test_method_test(self, client: Ark) -> None:
        webhook = client.webhooks.test(
            webhook_id="webhookId",
            event="MessageSent",
        )
        assert_matches_type(WebhookTestResponse, webhook, path=["response"])

    @parametrize
    def test_raw_response_test(self, client: Ark) -> None:
        response = client.webhooks.with_raw_response.test(
            webhook_id="webhookId",
            event="MessageSent",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = response.parse()
        assert_matches_type(WebhookTestResponse, webhook, path=["response"])

    @parametrize
    def test_streaming_response_test(self, client: Ark) -> None:
        with client.webhooks.with_streaming_response.test(
            webhook_id="webhookId",
            event="MessageSent",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = response.parse()
            assert_matches_type(WebhookTestResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_test(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            client.webhooks.with_raw_response.test(
                webhook_id="",
                event="MessageSent",
            )


class TestAsyncWebhooks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        )
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
            all_events=True,
            enabled=True,
            events=["MessageSent", "MessageDeliveryFailed", "MessageBounced"],
        )
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.create(
            name="My App Webhook",
            url="https://myapp.com/webhooks/email",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookCreateResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.retrieve(
            "webhookId",
        )
        assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.retrieve(
            "webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.retrieve(
            "webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookRetrieveResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.update(
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.update(
            webhook_id="webhookId",
            all_events=True,
            enabled=True,
            events=["string"],
            name="name",
            url="https://example.com",
        )
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.update(
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.update(
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookUpdateResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.update(
                webhook_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.list()
        assert_matches_type(WebhookListResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookListResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookListResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.delete(
            "webhookId",
        )
        assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.delete(
            "webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.delete(
            "webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookDeleteResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_list_deliveries(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.list_deliveries(
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    async def test_method_list_deliveries_with_all_params(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.list_deliveries(
            webhook_id="webhookId",
            after=0,
            before=0,
            event="MessageSent",
            page=1,
            per_page=1,
            success=True,
        )
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_list_deliveries(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.list_deliveries(
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_list_deliveries(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.list_deliveries(
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookListDeliveriesResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_deliveries(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.list_deliveries(
                webhook_id="",
            )

    @parametrize
    async def test_method_replay_delivery(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_replay_delivery(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_replay_delivery(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.replay_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookReplayDeliveryResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_replay_delivery(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.replay_delivery(
                delivery_id="deliveryId",
                webhook_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `delivery_id` but received ''"):
            await async_client.webhooks.with_raw_response.replay_delivery(
                delivery_id="",
                webhook_id="webhookId",
            )

    @parametrize
    async def test_method_retrieve_delivery(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )
        assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_delivery(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_delivery(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.retrieve_delivery(
            delivery_id="deliveryId",
            webhook_id="webhookId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookRetrieveDeliveryResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_delivery(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.retrieve_delivery(
                delivery_id="deliveryId",
                webhook_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `delivery_id` but received ''"):
            await async_client.webhooks.with_raw_response.retrieve_delivery(
                delivery_id="",
                webhook_id="webhookId",
            )

    @parametrize
    async def test_method_test(self, async_client: AsyncArk) -> None:
        webhook = await async_client.webhooks.test(
            webhook_id="webhookId",
            event="MessageSent",
        )
        assert_matches_type(WebhookTestResponse, webhook, path=["response"])

    @parametrize
    async def test_raw_response_test(self, async_client: AsyncArk) -> None:
        response = await async_client.webhooks.with_raw_response.test(
            webhook_id="webhookId",
            event="MessageSent",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        webhook = await response.parse()
        assert_matches_type(WebhookTestResponse, webhook, path=["response"])

    @parametrize
    async def test_streaming_response_test(self, async_client: AsyncArk) -> None:
        async with async_client.webhooks.with_streaming_response.test(
            webhook_id="webhookId",
            event="MessageSent",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            webhook = await response.parse()
            assert_matches_type(WebhookTestResponse, webhook, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_test(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `webhook_id` but received ''"):
            await async_client.webhooks.with_raw_response.test(
                webhook_id="",
                event="MessageSent",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import APIMeta as APIMeta
from .dns_record import DNSRecord as DNSRecord
from .track_domain import TrackDomain as TrackDomain
from .email_list_params import EmailListParams as EmailListParams
from .email_send_params import EmailSendParams as EmailSendParams
from .email_list_response import EmailListResponse as EmailListResponse
from .email_send_response import EmailSendResponse as EmailSendResponse
from .webhook_test_params import WebhookTestParams as WebhookTestParams
from .domain_create_params import DomainCreateParams as DomainCreateParams
from .domain_list_response import DomainListResponse as DomainListResponse
from .email_retry_response import EmailRetryResponse as EmailRetryResponse
from .email_retrieve_params import EmailRetrieveParams as EmailRetrieveParams
from .email_send_raw_params import EmailSendRawParams as EmailSendRawParams
from .webhook_create_params import WebhookCreateParams as WebhookCreateParams
from .webhook_list_response import WebhookListResponse as WebhookListResponse
from .webhook_test_response import WebhookTestResponse as WebhookTestResponse
from .webhook_update_params import WebhookUpdateParams as WebhookUpdateParams
from .domain_create_response import DomainCreateResponse as DomainCreateResponse
from .domain_delete_response import DomainDeleteResponse as DomainDeleteResponse
from .domain_verify_response import DomainVerifyResponse as DomainVerifyResponse
from .tracking_create_params import TrackingCreateParams as TrackingCreateParams
from .tracking_list_response import TrackingListResponse as TrackingListResponse
from .tracking_update_params import TrackingUpdateParams as TrackingUpdateParams
from .email_retrieve_response import EmailRetrieveResponse as EmailRetrieveResponse
from .email_send_batch_params import EmailSendBatchParams as EmailSendBatchParams
from .email_send_raw_response import EmailSendRawResponse as EmailSendRawResponse
from .suppression_list_params import SuppressionListParams as SuppressionListParams
from .webhook_create_response import WebhookCreateResponse as WebhookCreateResponse
from .webhook_delete_response import WebhookDeleteResponse as WebhookDeleteResponse
from .webhook_update_response import WebhookUpdateResponse as WebhookUpdateResponse
from .domain_retrieve_response import DomainRetrieveResponse as DomainRetrieveResponse
from .tracking_create_response import TrackingCreateResponse as TrackingCreateResponse
from .tracking_delete_response import TrackingDeleteResponse as TrackingDeleteResponse
from .tracking_update_response import TrackingUpdateResponse as TrackingUpdateResponse
from .tracking_verify_response import TrackingVerifyResponse as TrackingVerifyResponse
from .email_send_batch_response import EmailSendBatchResponse as EmailSendBatchResponse
from .suppression_create_params import SuppressionCreateParams as SuppressionCreateParams
from .suppression_list_response import SuppressionListResponse as SuppressionListResponse
from .webhook_retrieve_response import WebhookRetrieveResponse as WebhookRetrieveResponse
from .tracking_retrieve_response import TrackingRetrieveResponse as TrackingRetrieveResponse
from .suppression_create_response import SuppressionCreateResponse as SuppressionCreateResponse
from .suppression_delete_response import SuppressionDeleteResponse as SuppressionDeleteResponse
from .suppression_retrieve_response import SuppressionRetrieveResponse as SuppressionRetrieveResponse
from .suppression_bulk_create_params import SuppressionBulkCreateParams as SuppressionBulkCreateParams
from .webhook_list_deliveries_params import WebhookListDeliveriesParams as WebhookListDeliveriesParams
from .suppression_bulk_create_response import SuppressionBulkCreateResponse as SuppressionBulkCreateResponse
from .webhook_list_deliveries_response import WebhookListDeliveriesResponse as WebhookListDeliveriesResponse
from .webhook_replay_delivery_response import WebhookReplayDeliveryResponse as WebhookReplayDeliveryResponse
from .email_retrieve_deliveries_response import EmailRetrieveDeliveriesResponse as EmailRetrieveDeliveriesResponse
from .webhook_retrieve_delivery_response import WebhookRetrieveDeliveryResponse as WebhookRetrieveDeliveryResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["APIMeta"]


class APIMeta(BaseModel):
    request_id: str = FieldInfo(alias="requestId")
    """Unique request identifier for debugging and support"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["EmailRetrieveParams"]


class EmailRetrieveParams(TypedDict, total=False):
    expand: str
    """Comma-separated list of fields to include:

    - `content` - HTML and plain text body
    - `headers` - Email headers
    - `deliveries` - Delivery attempt history
    - `activity` - Opens and clicks
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["WebhookTestResponse", "Data"]


class Data(BaseModel):
    duration: int
    """Request duration in milliseconds"""

    event: str
    """Event type that was tested"""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP status code from the webhook endpoint"""

    success: bool
    """Whether the webhook endpoint responded with a 2xx status"""

    body: Optional[str] = None
    """Response body from the webhook endpoint (truncated if too long)"""

    error: Optional[str] = None
    """Error message if the request failed"""


class WebhookTestResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DNSRecord"]


class DNSRecord(BaseModel):
    name: str
    """DNS record name (hostname)"""

    type: Literal["TXT", "CNAME", "MX"]
    """DNS record type"""

    value: str
    """DNS record value"""

    status: Optional[Literal["OK", "Missing", "Invalid"]] = None
    """DNS verification status:

    - `OK` - Record is correctly configured
    - `Missing` - Record not found in DNS
    - `Invalid` - Record exists but has wrong value
    - `null` - Not yet checked
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["SuppressionCreateResponse", "Data"]


class Data(BaseModel):
    id: str
    """Suppression ID"""

    address: str

    created_at: datetime = FieldInfo(alias="createdAt")

    reason: Optional[str] = None
    """Reason for suppression"""


class SuppressionCreateResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import ArkError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import emails, domains, tracking, webhooks, suppressions
    from .resources.emails import EmailsResource, AsyncEmailsResource
    from .resources.domains import DomainsResource, AsyncDomainsResource
    from .resources.tracking import TrackingResource, AsyncTrackingResource
    from .resources.webhooks import WebhooksResource, AsyncWebhooksResource
    from .resources.suppressions import SuppressionsResource, AsyncSuppressionsResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Ark", "AsyncArk", "Client", "AsyncClient"]


class Ark(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Ark client instance.

        This automatically infers the `api_key` argument from the `ARK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ARK_API_KEY")
        if api_key is None:
            raise ArkError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ARK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ARK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.arkhq.io/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def emails(self) -> EmailsResource:
        from .resources.emails import EmailsResource

        return EmailsResource(self)

    @cached_property
    def domains(self) -> DomainsResource:
        from .resources.domains import DomainsResource

        return DomainsResource(self)

    @cached_property
    def suppressions(self) -> SuppressionsResource:
        from .resources.suppressions import SuppressionsResource

        return SuppressionsResource(self)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        from .resources.webhooks import WebhooksResource

        return WebhooksResource(self)

    @cached_property
    def tracking(self) -> TrackingResource:
        from .resources.tracking import TrackingResource

        return TrackingResource(self)

    @cached_property
    def with_raw_response(self) -> ArkWithRawResponse:
        return ArkWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ArkWithStreamedResponse:
        return ArkWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncArk(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncArk client instance.

        This automatically infers the `api_key` argument from the `ARK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ARK_API_KEY")
        if api_key is None:
            raise ArkError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ARK_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("ARK_BASE_URL")
        if base_url is None:
            base_url = f"https://api.arkhq.io/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def emails(self) -> AsyncEmailsResource:
        from .resources.emails import AsyncEmailsResource

        return AsyncEmailsResource(self)

    @cached_property
    def domains(self) -> AsyncDomainsResource:
        from .resources.domains import AsyncDomainsResource

        return AsyncDomainsResource(self)

    @cached_property
    def suppressions(self) -> AsyncSuppressionsResource:
        from .resources.suppressions import AsyncSuppressionsResource

        return AsyncSuppressionsResource(self)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        from .resources.webhooks import AsyncWebhooksResource

        return AsyncWebhooksResource(self)

    @cached_property
    def tracking(self) -> AsyncTrackingResource:
        from .resources.tracking import AsyncTrackingResource

        return AsyncTrackingResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncArkWithRawResponse:
        return AsyncArkWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncArkWithStreamedResponse:
        return AsyncArkWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ArkWithRawResponse:
    _client: Ark

    def __init__(self, client: Ark) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.EmailsResourceWithRawResponse:
        from .resources.emails import EmailsResourceWithRawResponse

        return EmailsResourceWithRawResponse(self._client.emails)

    @cached_property
    def domains(self) -> domains.DomainsResourceWithRawResponse:
        from .resources.domains import DomainsResourceWithRawResponse

        return DomainsResourceWithRawResponse(self._client.domains)

    @cached_property
    def suppressions(self) -> suppressions.SuppressionsResourceWithRawResponse:
        from .resources.suppressions import SuppressionsResourceWithRawResponse

        return SuppressionsResourceWithRawResponse(self._client.suppressions)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithRawResponse:
        from .resources.webhooks import WebhooksResourceWithRawResponse

        return WebhooksResourceWithRawResponse(self._client.webhooks)

    @cached_property
    def tracking(self) -> tracking.TrackingResourceWithRawResponse:
        from .resources.tracking import TrackingResourceWithRawResponse

        return TrackingResourceWithRawResponse(self._client.tracking)


class AsyncArkWithRawResponse:
    _client: AsyncArk

    def __init__(self, client: AsyncArk) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.AsyncEmailsResourceWithRawResponse:
        from .resources.emails import AsyncEmailsResourceWithRawResponse

        return AsyncEmailsResourceWithRawResponse(self._client.emails)

    @cached_property
    def domains(self) -> domains.AsyncDomainsResourceWithRawResponse:
        from .resources.domains import AsyncDomainsResourceWithRawResponse

        return AsyncDomainsResourceWithRawResponse(self._client.domains)

    @cached_property
    def suppressions(self) -> suppressions.AsyncSuppressionsResourceWithRawResponse:
        from .resources.suppressions import AsyncSuppressionsResourceWithRawResponse

        return AsyncSuppressionsResourceWithRawResponse(self._client.suppressions)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithRawResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithRawResponse

        return AsyncWebhooksResourceWithRawResponse(self._client.webhooks)

    @cached_property
    def tracking(self) -> tracking.AsyncTrackingResourceWithRawResponse:
        from .resources.tracking import AsyncTrackingResourceWithRawResponse

        return AsyncTrackingResourceWithRawResponse(self._client.tracking)


class ArkWithStreamedResponse:
    _client: Ark

    def __init__(self, client: Ark) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.EmailsResourceWithStreamingResponse:
        from .resources.emails import EmailsResourceWithStreamingResponse

        return EmailsResourceWithStreamingResponse(self._client.emails)

    @cached_property
    def domains(self) -> domains.DomainsResourceWithStreamingResponse:
        from .resources.domains import DomainsResourceWithStreamingResponse

        return DomainsResourceWithStreamingResponse(self._client.domains)

    @cached_property
    def suppressions(self) -> suppressions.SuppressionsResourceWithStreamingResponse:
        from .resources.suppressions import SuppressionsResourceWithStreamingResponse

        return SuppressionsResourceWithStreamingResponse(self._client.suppressions)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithStreamingResponse:
        from .resources.webhooks import WebhooksResourceWithStreamingResponse

        return WebhooksResourceWithStreamingResponse(self._client.webhooks)

    @cached_property
    def tracking(self) -> tracking.TrackingResourceWithStreamingResponse:
        from .resources.tracking import TrackingResourceWithStreamingResponse

        return TrackingResourceWithStreamingResponse(self._client.tracking)


class AsyncArkWithStreamedResponse:
    _client: AsyncArk

    def __init__(self, client: AsyncArk) -> None:
        self._client = client

    @cached_property
    def emails(self) -> emails.AsyncEmailsResourceWithStreamingResponse:
        from .resources.emails import AsyncEmailsResourceWithStreamingResponse

        return AsyncEmailsResourceWithStreamingResponse(self._client.emails)

    @cached_property
    def domains(self) -> domains.AsyncDomainsResourceWithStreamingResponse:
        from .resources.domains import AsyncDomainsResourceWithStreamingResponse

        return AsyncDomainsResourceWithStreamingResponse(self._client.domains)

    @cached_property
    def suppressions(self) -> suppressions.AsyncSuppressionsResourceWithStreamingResponse:
        from .resources.suppressions import AsyncSuppressionsResourceWithStreamingResponse

        return AsyncSuppressionsResourceWithStreamingResponse(self._client.suppressions)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithStreamingResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithStreamingResponse

        return AsyncWebhooksResourceWithStreamingResponse(self._client.webhooks)

    @cached_property
    def tracking(self) -> tracking.AsyncTrackingResourceWithStreamingResponse:
        from .resources.tracking import AsyncTrackingResourceWithStreamingResponse

        return AsyncTrackingResourceWithStreamingResponse(self._client.tracking)


Client = Ark

AsyncClient = AsyncArk



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["SuppressionBulkCreateResponse", "Data"]


class Data(BaseModel):
    added: int
    """Newly suppressed addresses"""

    failed: int
    """Invalid addresses skipped"""

    total_requested: int = FieldInfo(alias="totalRequested")
    """Total addresses in request"""

    updated: int
    """Already suppressed addresses (updated reason)"""


class SuppressionBulkCreateResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from ark import Ark, AsyncArk
from ark.types import (
    EmailListResponse,
    EmailSendResponse,
    EmailRetryResponse,
    EmailSendRawResponse,
    EmailRetrieveResponse,
    EmailSendBatchResponse,
    EmailRetrieveDeliveriesResponse,
)
from tests.utils import assert_matches_type
from ark.pagination import SyncPageNumberPagination, AsyncPageNumberPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEmails:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Ark) -> None:
        email = client.emails.retrieve(
            email_id="emailId",
        )
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: Ark) -> None:
        email = client.emails.retrieve(
            email_id="emailId",
            expand="content,deliveries",
        )
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Ark) -> None:
        response = client.emails.with_raw_response.retrieve(
            email_id="emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Ark) -> None:
        with client.emails.with_streaming_response.retrieve(
            email_id="emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailRetrieveResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            client.emails.with_raw_response.retrieve(
                email_id="",
            )

    @parametrize
    def test_method_list(self, client: Ark) -> None:
        email = client.emails.list()
        assert_matches_type(SyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Ark) -> None:
        email = client.emails.list(
            after="after",
            before="before",
            from_="dev@stainless.com",
            page=1,
            per_page=1,
            status="pending",
            tag="tag",
            to="dev@stainless.com",
        )
        assert_matches_type(SyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Ark) -> None:
        response = client.emails.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(SyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Ark) -> None:
        with client.emails.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(SyncPageNumberPagination[EmailListResponse], email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve_deliveries(self, client: Ark) -> None:
        email = client.emails.retrieve_deliveries(
            "emailId",
        )
        assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

    @parametrize
    def test_raw_response_retrieve_deliveries(self, client: Ark) -> None:
        response = client.emails.with_raw_response.retrieve_deliveries(
            "emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_deliveries(self, client: Ark) -> None:
        with client.emails.with_streaming_response.retrieve_deliveries(
            "emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_deliveries(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            client.emails.with_raw_response.retrieve_deliveries(
                "",
            )

    @parametrize
    def test_method_retry(self, client: Ark) -> None:
        email = client.emails.retry(
            "emailId",
        )
        assert_matches_type(EmailRetryResponse, email, path=["response"])

    @parametrize
    def test_raw_response_retry(self, client: Ark) -> None:
        response = client.emails.with_raw_response.retry(
            "emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailRetryResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_retry(self, client: Ark) -> None:
        with client.emails.with_streaming_response.retry(
            "emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailRetryResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retry(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            client.emails.with_raw_response.retry(
                "",
            )

    @parametrize
    def test_method_send(self, client: Ark) -> None:
        email = client.emails.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        )
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    def test_method_send_with_all_params(self, client: Ark) -> None:
        email = client.emails.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
            attachments=[
                {
                    "content": "content",
                    "content_type": "application/pdf",
                    "filename": "filename",
                }
            ],
            bcc=["dev@stainless.com"],
            cc=["dev@stainless.com"],
            headers={"foo": "string"},
            html="<h1>Welcome!</h1><p>Thanks for signing up.</p>",
            metadata={
                "user_id": "usr_123",
                "campaign": "onboarding",
            },
            reply_to="dev@stainless.com",
            tag="tag",
            text="text",
            idempotency_key="user_123_order_456",
        )
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    def test_raw_response_send(self, client: Ark) -> None:
        response = client.emails.with_raw_response.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_send(self, client: Ark) -> None:
        with client.emails.with_streaming_response.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailSendResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_send_batch(self, client: Ark) -> None:
        email = client.emails.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        )
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    def test_method_send_batch_with_all_params(self, client: Ark) -> None:
        email = client.emails.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                    "html": "<p>Hi Alice, your order is ready!</p>",
                    "metadata": {
                        "user_id": "usr_123456",
                        "order_id": "ord_789012",
                        "campaign": "welcome_series",
                    },
                    "tag": "order-ready",
                    "text": "text",
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                    "html": "<p>Hi Bob, your order is ready!</p>",
                    "metadata": {
                        "user_id": "usr_123456",
                        "order_id": "ord_789012",
                        "campaign": "welcome_series",
                    },
                    "tag": "order-ready",
                    "text": "text",
                },
            ],
            from_="notifications@myapp.com",
            idempotency_key="user_123_order_456",
        )
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    def test_raw_response_send_batch(self, client: Ark) -> None:
        response = client.emails.with_raw_response.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_send_batch(self, client: Ark) -> None:
        with client.emails.with_streaming_response.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailSendBatchResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_send_raw(self, client: Ark) -> None:
        email = client.emails.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        )
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    def test_method_send_raw_with_all_params(self, client: Ark) -> None:
        email = client.emails.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
            bounce=True,
        )
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    def test_raw_response_send_raw(self, client: Ark) -> None:
        response = client.emails.with_raw_response.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = response.parse()
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    def test_streaming_response_send_raw(self, client: Ark) -> None:
        with client.emails.with_streaming_response.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = response.parse()
            assert_matches_type(EmailSendRawResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncEmails:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.retrieve(
            email_id="emailId",
        )
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.retrieve(
            email_id="emailId",
            expand="content,deliveries",
        )
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.retrieve(
            email_id="emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailRetrieveResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.retrieve(
            email_id="emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailRetrieveResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            await async_client.emails.with_raw_response.retrieve(
                email_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.list()
        assert_matches_type(AsyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.list(
            after="after",
            before="before",
            from_="dev@stainless.com",
            page=1,
            per_page=1,
            status="pending",
            tag="tag",
            to="dev@stainless.com",
        )
        assert_matches_type(AsyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(AsyncPageNumberPagination[EmailListResponse], email, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(AsyncPageNumberPagination[EmailListResponse], email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve_deliveries(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.retrieve_deliveries(
            "emailId",
        )
        assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_deliveries(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.retrieve_deliveries(
            "emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_deliveries(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.retrieve_deliveries(
            "emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailRetrieveDeliveriesResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_deliveries(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            await async_client.emails.with_raw_response.retrieve_deliveries(
                "",
            )

    @parametrize
    async def test_method_retry(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.retry(
            "emailId",
        )
        assert_matches_type(EmailRetryResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_retry(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.retry(
            "emailId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailRetryResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_retry(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.retry(
            "emailId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailRetryResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retry(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email_id` but received ''"):
            await async_client.emails.with_raw_response.retry(
                "",
            )

    @parametrize
    async def test_method_send(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        )
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    async def test_method_send_with_all_params(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
            attachments=[
                {
                    "content": "content",
                    "content_type": "application/pdf",
                    "filename": "filename",
                }
            ],
            bcc=["dev@stainless.com"],
            cc=["dev@stainless.com"],
            headers={"foo": "string"},
            html="<h1>Welcome!</h1><p>Thanks for signing up.</p>",
            metadata={
                "user_id": "usr_123",
                "campaign": "onboarding",
            },
            reply_to="dev@stainless.com",
            tag="tag",
            text="text",
            idempotency_key="user_123_order_456",
        )
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_send(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailSendResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_send(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.send(
            from_="Acme <hello@acme.com>",
            subject="Hello World",
            to=["user@example.com"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailSendResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_send_batch(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        )
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    async def test_method_send_batch_with_all_params(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                    "html": "<p>Hi Alice, your order is ready!</p>",
                    "metadata": {
                        "user_id": "usr_123456",
                        "order_id": "ord_789012",
                        "campaign": "welcome_series",
                    },
                    "tag": "order-ready",
                    "text": "text",
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                    "html": "<p>Hi Bob, your order is ready!</p>",
                    "metadata": {
                        "user_id": "usr_123456",
                        "order_id": "ord_789012",
                        "campaign": "welcome_series",
                    },
                    "tag": "order-ready",
                    "text": "text",
                },
            ],
            from_="notifications@myapp.com",
            idempotency_key="user_123_order_456",
        )
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_send_batch(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailSendBatchResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_send_batch(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.send_batch(
            emails=[
                {
                    "subject": "Hello Alice",
                    "to": ["alice@example.com"],
                },
                {
                    "subject": "Hello Bob",
                    "to": ["bob@example.com"],
                },
            ],
            from_="notifications@myapp.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailSendBatchResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_send_raw(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        )
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    async def test_method_send_raw_with_all_params(self, async_client: AsyncArk) -> None:
        email = await async_client.emails.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
            bounce=True,
        )
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    async def test_raw_response_send_raw(self, async_client: AsyncArk) -> None:
        response = await async_client.emails.with_raw_response.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        email = await response.parse()
        assert_matches_type(EmailSendRawResponse, email, path=["response"])

    @parametrize
    async def test_streaming_response_send_raw(self, async_client: AsyncArk) -> None:
        async with async_client.emails.with_streaming_response.send_raw(
            data="data",
            mail_from="dev@stainless.com",
            rcpt_to=["dev@stainless.com"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            email = await response.parse()
            assert_matches_type(EmailSendRawResponse, email, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["EmailSendRawParams"]


class EmailSendRawParams(TypedDict, total=False):
    data: Required[str]
    """Base64-encoded RFC 2822 message"""

    mail_from: Required[Annotated[str, PropertyInfo(alias="mailFrom")]]
    """Envelope sender address"""

    rcpt_to: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="rcptTo")]]
    """Envelope recipient addresses"""

    bounce: Optional[bool]
    """Whether this is a bounce message (accepts null)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["WebhookListDeliveriesResponse", "Data"]


class Data(BaseModel):
    """Summary of a webhook delivery attempt"""

    id: str
    """Unique delivery ID (UUID)"""

    attempt: int
    """Attempt number (1 for first attempt, increments with retries)"""

    event: Literal[
        "MessageSent",
        "MessageDelayed",
        "MessageDeliveryFailed",
        "MessageHeld",
        "MessageBounced",
        "MessageLinkClicked",
        "MessageLoaded",
        "DomainDNSError",
    ]
    """Event type that triggered this delivery"""

    status_code: Optional[int] = FieldInfo(alias="statusCode", default=None)
    """HTTP status code returned by the endpoint (null if connection failed)"""

    success: bool
    """Whether the delivery was successful (2xx response)"""

    timestamp: datetime
    """When this delivery attempt occurred"""

    url: str
    """URL the webhook was delivered to"""

    webhook_id: str = FieldInfo(alias="webhookId")
    """ID of the webhook this delivery belongs to"""

    will_retry: bool = FieldInfo(alias="willRetry")
    """Whether this delivery will be retried (true if failed and retries remaining)"""


class WebhookListDeliveriesResponse(BaseModel):
    """Paginated list of webhook delivery attempts"""

    data: List[Data]

    meta: APIMeta

    page: int
    """Current page number"""

    per_page: int = FieldInfo(alias="perPage")
    """Items per page"""

    total: int
    """Total number of deliveries matching the filter"""

    total_pages: int = FieldInfo(alias="totalPages")
    """Total number of pages"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from ark import Ark, AsyncArk
from ark.types import (
    SuppressionListResponse,
    SuppressionCreateResponse,
    SuppressionDeleteResponse,
    SuppressionRetrieveResponse,
    SuppressionBulkCreateResponse,
)
from tests.utils import assert_matches_type
from ark.pagination import SyncPageNumberPagination, AsyncPageNumberPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSuppressions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Ark) -> None:
        suppression = client.suppressions.create(
            address="user@example.com",
        )
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Ark) -> None:
        suppression = client.suppressions.create(
            address="user@example.com",
            reason="user requested removal",
        )
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Ark) -> None:
        response = client.suppressions.with_raw_response.create(
            address="user@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = response.parse()
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Ark) -> None:
        with client.suppressions.with_streaming_response.create(
            address="user@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = response.parse()
            assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Ark) -> None:
        suppression = client.suppressions.retrieve(
            "dev@stainless.com",
        )
        assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Ark) -> None:
        response = client.suppressions.with_raw_response.retrieve(
            "dev@stainless.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = response.parse()
        assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Ark) -> None:
        with client.suppressions.with_streaming_response.retrieve(
            "dev@stainless.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = response.parse()
            assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email` but received ''"):
            client.suppressions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Ark) -> None:
        suppression = client.suppressions.list()
        assert_matches_type(SyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Ark) -> None:
        suppression = client.suppressions.list(
            page=0,
            per_page=100,
        )
        assert_matches_type(SyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Ark) -> None:
        response = client.suppressions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = response.parse()
        assert_matches_type(SyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Ark) -> None:
        with client.suppressions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = response.parse()
            assert_matches_type(SyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Ark) -> None:
        suppression = client.suppressions.delete(
            "dev@stainless.com",
        )
        assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Ark) -> None:
        response = client.suppressions.with_raw_response.delete(
            "dev@stainless.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = response.parse()
        assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Ark) -> None:
        with client.suppressions.with_streaming_response.delete(
            "dev@stainless.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = response.parse()
            assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Ark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email` but received ''"):
            client.suppressions.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_bulk_create(self, client: Ark) -> None:
        suppression = client.suppressions.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        )
        assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

    @parametrize
    def test_raw_response_bulk_create(self, client: Ark) -> None:
        response = client.suppressions.with_raw_response.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = response.parse()
        assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

    @parametrize
    def test_streaming_response_bulk_create(self, client: Ark) -> None:
        with client.suppressions.with_streaming_response.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = response.parse()
            assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSuppressions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.create(
            address="user@example.com",
        )
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.create(
            address="user@example.com",
            reason="user requested removal",
        )
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncArk) -> None:
        response = await async_client.suppressions.with_raw_response.create(
            address="user@example.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = await response.parse()
        assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncArk) -> None:
        async with async_client.suppressions.with_streaming_response.create(
            address="user@example.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = await response.parse()
            assert_matches_type(SuppressionCreateResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.retrieve(
            "dev@stainless.com",
        )
        assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncArk) -> None:
        response = await async_client.suppressions.with_raw_response.retrieve(
            "dev@stainless.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = await response.parse()
        assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncArk) -> None:
        async with async_client.suppressions.with_streaming_response.retrieve(
            "dev@stainless.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = await response.parse()
            assert_matches_type(SuppressionRetrieveResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email` but received ''"):
            await async_client.suppressions.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.list()
        assert_matches_type(AsyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.list(
            page=0,
            per_page=100,
        )
        assert_matches_type(AsyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncArk) -> None:
        response = await async_client.suppressions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = await response.parse()
        assert_matches_type(AsyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncArk) -> None:
        async with async_client.suppressions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = await response.parse()
            assert_matches_type(AsyncPageNumberPagination[SuppressionListResponse], suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.delete(
            "dev@stainless.com",
        )
        assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncArk) -> None:
        response = await async_client.suppressions.with_raw_response.delete(
            "dev@stainless.com",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = await response.parse()
        assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncArk) -> None:
        async with async_client.suppressions.with_streaming_response.delete(
            "dev@stainless.com",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = await response.parse()
            assert_matches_type(SuppressionDeleteResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncArk) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `email` but received ''"):
            await async_client.suppressions.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_bulk_create(self, async_client: AsyncArk) -> None:
        suppression = await async_client.suppressions.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        )
        assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

    @parametrize
    async def test_raw_response_bulk_create(self, async_client: AsyncArk) -> None:
        response = await async_client.suppressions.with_raw_response.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        suppression = await response.parse()
        assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

    @parametrize
    async def test_streaming_response_bulk_create(self, async_client: AsyncArk) -> None:
        async with async_client.suppressions.with_streaming_response.bulk_create(
            suppressions=[{"address": "dev@stainless.com"}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            suppression = await response.parse()
            assert_matches_type(SuppressionBulkCreateResponse, suppression, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SuppressionListParams"]


class SuppressionListParams(TypedDict, total=False):
    page: int

    per_page: Annotated[int, PropertyInfo(alias="perPage")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TrackDomain", "DNSRecord"]


class DNSRecord(BaseModel):
    """Required DNS record configuration"""

    name: Optional[str] = None
    """DNS record name"""

    type: Optional[str] = None
    """DNS record type"""

    value: Optional[str] = None
    """DNS record value (target)"""


class TrackDomain(BaseModel):
    id: str
    """Track domain ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """When the track domain was created"""

    dns_ok: bool = FieldInfo(alias="dnsOk")
    """Whether DNS is correctly configured"""

    domain_id: str = FieldInfo(alias="domainId")
    """ID of the parent sending domain"""

    full_name: str = FieldInfo(alias="fullName")
    """Full domain name"""

    name: str
    """Subdomain name"""

    ssl_enabled: bool = FieldInfo(alias="sslEnabled")
    """Whether SSL is enabled for tracking URLs"""

    track_clicks: bool = FieldInfo(alias="trackClicks")
    """Whether click tracking is enabled"""

    track_opens: bool = FieldInfo(alias="trackOpens")
    """Whether open tracking is enabled"""

    dns_checked_at: Optional[datetime] = FieldInfo(alias="dnsCheckedAt", default=None)
    """When DNS was last checked"""

    dns_error: Optional[str] = FieldInfo(alias="dnsError", default=None)
    """DNS error message if verification failed"""

    dns_record: Optional[DNSRecord] = FieldInfo(alias="dnsRecord", default=None)
    """Required DNS record configuration"""

    dns_status: Optional[Literal["ok", "missing", "invalid"]] = FieldInfo(alias="dnsStatus", default=None)
    """Current DNS verification status"""

    excluded_click_domains: Optional[str] = FieldInfo(alias="excludedClickDomains", default=None)
    """Domains excluded from click tracking"""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)
    """When the track domain was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .dns_record import DNSRecord
from .shared.api_meta import APIMeta

__all__ = ["DomainCreateResponse", "Data", "DataDNSRecords"]


class DataDNSRecords(BaseModel):
    dkim: DNSRecord

    return_path: DNSRecord = FieldInfo(alias="returnPath")

    spf: DNSRecord


class Data(BaseModel):
    id: str
    """Domain ID"""

    created_at: datetime = FieldInfo(alias="createdAt")

    dns_records: DataDNSRecords = FieldInfo(alias="dnsRecords")

    name: str
    """Domain name"""

    uuid: str

    verified: bool
    """Whether DNS is verified"""

    verified_at: Optional[datetime] = FieldInfo(alias="verifiedAt", default=None)
    """When the domain was verified (null if not verified)"""


class DomainCreateResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel
from .track_domain import TrackDomain
from .shared.api_meta import APIMeta

__all__ = ["TrackingRetrieveResponse"]


class TrackingRetrieveResponse(BaseModel):
    data: TrackDomain

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["WebhookListResponse", "Data", "DataWebhook"]


class DataWebhook(BaseModel):
    id: str
    """Webhook ID"""

    enabled: bool

    events: List[str]

    name: str

    url: str


class Data(BaseModel):
    webhooks: List[DataWebhook]


class WebhookListResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["EmailListResponse"]


class EmailListResponse(BaseModel):
    id: str
    """Internal message ID"""

    token: str

    from_: str = FieldInfo(alias="from")

    status: Literal["pending", "sent", "softfail", "hardfail", "bounced", "held"]
    """Current delivery status:

    - `pending` - Email accepted, waiting to be processed
    - `sent` - Email transmitted to recipient's mail server
    - `softfail` - Temporary delivery failure, will retry
    - `hardfail` - Permanent delivery failure
    - `bounced` - Email bounced back
    - `held` - Held for manual review
    """

    subject: str

    timestamp: float

    timestamp_iso: datetime = FieldInfo(alias="timestampIso")

    to: str

    tag: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["DomainListResponse", "Data", "DataDomain"]


class DataDomain(BaseModel):
    id: str
    """Domain ID"""

    dns_ok: bool = FieldInfo(alias="dnsOk")

    name: str

    verified: bool


class Data(BaseModel):
    domains: List[DataDomain]


class DomainListResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Required, TypedDict

__all__ = ["SuppressionBulkCreateParams", "Suppression"]


class SuppressionBulkCreateParams(TypedDict, total=False):
    suppressions: Required[Iterable[Suppression]]


class Suppression(TypedDict, total=False):
    address: Required[str]

    reason: Optional[str]
    """Reason for suppression (accepts null)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.api_meta import APIMeta

__all__ = ["WebhookRetrieveResponse", "Data"]


class Data(BaseModel):
    id: str
    """Webhook ID"""

    all_events: bool = FieldInfo(alias="allEvents")
    """Whether subscribed to all events"""

    created_at: datetime = FieldInfo(alias="createdAt")

    enabled: bool
    """Whether the webhook is active"""

    events: List[
        Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ]
    ]
    """Subscribed events"""

    name: str
    """Webhook name for identification"""

    url: str
    """Webhook endpoint URL"""

    uuid: str


class WebhookRetrieveResponse(BaseModel):
    data: Data

    meta: APIMeta

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["WebhookTestParams"]


class WebhookTestParams(TypedDict, total=False):
    event: Required[
        Literal[
            "MessageSent",
            "MessageDelayed",
            "MessageDeliveryFailed",
            "MessageHeld",
            "MessageBounced",
            "MessageLinkClicked",
            "MessageLoaded",
            "DomainDNSError",
        ]
    ]
    """Event type to simulate"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["EmailListParams"]


class EmailListParams(TypedDict, total=False):
    after: str
    """Return emails sent after this timestamp (Unix seconds or ISO 8601)"""

    before: str
    """Return emails sent before this timestamp"""

    from_: Annotated[str, PropertyInfo(alias="from")]
    """Filter by sender email address"""

    page: int
    """Page number (starts at 1)"""

    per_page: Annotated[int, PropertyInfo(alias="perPage")]
    """Results per page (max 100)"""

    status: Literal["pending", "sent", "softfail", "hardfail", "bounced", "held"]
    """Filter by delivery status:

    - `pending` - Email accepted, waiting to be processed
    - `sent` - Email transmitted to recipient's mail server
    - `softfail` - Temporary delivery failure, will retry
    - `hardfail` - Permanent delivery failure
    - `bounced` - Email bounced back
    - `held` - Held for manual review
    """

    tag: str
    """Filter by tag"""

    to: str
    """Filter by recipient email address"""
